import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { DataSchedulingCreatedEvent } from '../../entities/event/data-scheduling-created.event';
import {
  DataSchedulingLogEntity,
  SCHEDULING_LOG_ACTION_ENUM,
  SCHEDULING_LOG_TYPE_ENUM,
} from '../../entities/data-scheduling.entity';
import { decryptionTotal } from '../../../infrastructure/helpers';
import { DataSchedulingLogModel } from '../../../data/models/data-scheduling-log.model';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Logger } from '@nestjs/common';

@EventsHandler(DataSchedulingCreatedEvent)
export class DataSchedulingCreatedHandler
  implements IEventHandler<DataSchedulingCreatedEvent>
{
  private readonly logger = new Logger(DataSchedulingCreatedHandler.name);

  constructor(
    @InjectRepository(DataSchedulingLogModel)
    private repository: Repository<DataSchedulingLogModel>,
  ) {}

  async handle(event: DataSchedulingCreatedEvent) {
    const data = event?.data?.data;
    const totalPercentage = decryptionTotal(data?.indexing_key);

    const scheduleName = data?.name || 'a new schedule';
    const description = `<p><b>${data.creator_name}</b> created <b>${scheduleName}</b> schedule from <b><i>${data?.schedule_date_from}</i></b> to <b><i>${data.schedule_date_to}</i></b> with a total percentage of <b><i>${totalPercentage}%</i></b>.</p>`;

    const payload: DataSchedulingLogEntity = {
      type: SCHEDULING_LOG_TYPE_ENUM.DATA_SCHEDULING,
      action: SCHEDULING_LOG_ACTION_ENUM.CREATE,
      log_created_at: new Date().getTime(),

      data_id: data?.id,
      name: data?.name,
      indexing_key: data?.indexing_key,
      schedule_date_from: data?.schedule_date_from,
      schedule_date_to: data?.schedule_date_to,

      status: data?.status,
      creator_id: data?.creator_id,
      creator_name: data?.creator_name,
      editor_id: data?.editor_id,
      editor_name: data?.editor_name,
      created_at: data?.created_at,
      updated_at: data?.updated_at,
      description: description,
    };

    await this.repository.save(payload as any);
    this.logger.verbose(
      `[SCHEDULING LOG] Create data for ID: ${payload.data_id}`,
    );
  }
}
