import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { DataSchedulingDeletedEvent } from '../../entities/event/data-scheduling-deleted.event';
import {
  DataSchedulingLogEntity,
  SCHEDULING_LOG_ACTION_ENUM,
  SCHEDULING_LOG_TYPE_ENUM,
} from '../../entities/data-scheduling.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { DataSchedulingLogModel } from '../../../data/models/data-scheduling-log.model';
import { Repository } from 'typeorm';
import { Logger } from '@nestjs/common';

@EventsHandler(DataSchedulingDeletedEvent)
export class DataSchedulingDeletedHandler
  implements IEventHandler<DataSchedulingDeletedEvent>
{
  private readonly logger = new Logger(DataSchedulingDeletedHandler.name);

  constructor(
    @InjectRepository(DataSchedulingLogModel)
    private repository: Repository<DataSchedulingLogModel>,
  ) {}

  async handle(event: DataSchedulingDeletedEvent) {
    const deletedData = event?.data?.data;
    const user = event?.data?.user;

    const deleterName =
      user?.name ||
      deletedData?.editor_name ||
      deletedData?.creator_name ||
      'System';

    const scheduleName = deletedData?.name || 'an item';

    const description = `<p><b>${deleterName}</b> deleted schedule: <b>${scheduleName}</b>.</p>`;

    const payload: DataSchedulingLogEntity = {
      type: SCHEDULING_LOG_TYPE_ENUM.DATA_SCHEDULING,
      action: SCHEDULING_LOG_ACTION_ENUM.DELETE,
      log_created_at: new Date().getTime(),

      data_id: deletedData?.id,
      name: deletedData?.name,
      indexing_key: deletedData?.indexing_key,
      schedule_date_from: deletedData?.schedule_date_from,
      schedule_date_to: deletedData?.schedule_date_to,
      status: deletedData?.status,

      creator_id: deletedData?.creator_id,
      creator_name: deletedData?.creator_name,
      editor_id: deletedData?.editor_id,
      editor_name: deletedData?.editor_name,
      created_at: deletedData?.created_at,
      updated_at: deletedData?.updated_at,
      description: description,
    };

    await this.repository.save(payload as any);
    this.logger.verbose(
      `[SCHEDULING LOG] Delete data for ID: ${payload.data_id}`,
    );
  }
}
